"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var orderBy_1 = tslib_1.__importDefault(require("lodash/orderBy"));
var mobx_1 = require("mobx");
var mobx_react_1 = require("mobx-react");
var http_1 = require("../helper/http");
var types_1 = require("../@typings/types");
var pagination_1 = require("../helper/pagination");
var field_1 = tslib_1.__importDefault(require("./models/field"));
var header_1 = tslib_1.__importDefault(require("./models/header"));
function useRootStore(props) {
    var observableProps = mobx_react_1.useAsObservableSource(props);
    return mobx_react_1.useLocalStore(function () { return ({
        props: observableProps,
        inProgress: false,
        _loadedData: undefined,
        currentPage: 0,
        selectedItems: {},
        sorting: {},
        error: undefined,
        searchQuery: '',
        filterHandlers: function () {
            return [
                this.searchFilter,
                this.sortFilter
            ];
        },
        searchFilter: function (data) {
            var _this = this;
            if (this.searchQuery.length) {
                data = data.filter(function (row) { return _this.fields.some(function (field) {
                    var searchable = _this.headers && _this.headers[field.index].searchable;
                    return searchable && row[field.property].toString().toLowerCase()
                        .indexOf(_this.searchQuery.toLowerCase()) !== -1;
                }); });
            }
            return data;
        },
        sortFilter: function (data) {
            if (Object.keys(this.sorting).length) {
                return orderBy_1.default(data, Object.keys(this.sorting), Object.values(this.sorting).map(function (v) { return v.order; }));
            }
            return data;
        },
        paginateFilter: function (data) {
            var _data = data;
            if (this.props.pagination && _data.length > this.props.pagination.pageSize && this.props.pagination.pageSize) {
                _data = pagination_1.paginate(data, this.props.pagination.pageSize, this.currentPage);
            }
            return { data: _data, pageCount: pagination_1.calculatePageCount(data, this.props.pagination) };
        },
        search: function (query) {
            this.searchQuery = query;
            this.currentPage = 0;
        },
        select: function (row) {
            if (this.props.selectMode) {
                var indexFieldValue = row[this.props.indexField];
                if (!this.selectedItems.hasOwnProperty(indexFieldValue)) {
                    if (this.props.selectMode === types_1.SelectModes.single) {
                        this.selectedItems = {};
                    }
                    this.selectedItems[indexFieldValue] = row;
                }
                else {
                    delete this.selectedItems[indexFieldValue];
                }
                if (this.props.onSelect) {
                    this.props.onSelect(Object.values(this.selectedItems));
                }
            }
        },
        get isLoading() {
            return this.inProgress && !this.error;
        },
        sort: function (header) {
            var _a;
            if (this.isLoading) {
                return;
            }
            var property = this.fields[header.index].property;
            var headerName = header.name;
            if (this.sorting[property]) {
                this.sorting[property].order = this.sorting[property].order === types_1.SortingValues.ASC ? types_1.SortingValues.DESC
                    : types_1.SortingValues.ASC;
            }
            else if (this.props.sorting === types_1.SortingModes.simple) {
                this.sorting = (_a = {},
                    _a[property] = {
                        headerName: headerName,
                        order: types_1.SortingValues.ASC
                    },
                    _a);
            }
            else {
                this.sorting[property] = {
                    headerName: headerName,
                    order: types_1.SortingValues.ASC
                };
            }
            this.currentPage = 0;
        },
        removeFromSorting: function (property) {
            delete this.sorting[property];
            this.currentPage = 0;
        },
        editCell: function (newValue, record, model) {
            if (record[model.property].toString() === newValue) {
                return;
            }
            record[model.property] = newValue;
            if (this.props.onEdit) {
                this.props.onEdit(newValue, model.property, mobx_1.toJS(record));
            }
        },
        isEditableField: function (field) {
            if (this.headers && this.headers.length) {
                var header = this.headers[field.index];
                if ((header.editable || this.props.editable) && field.renderHandler) {
                    console.warn("Please avoid to use editable mode with custom render method for property: " + field.property);
                }
                if (typeof header.editable !== "undefined") {
                    return header.editable;
                }
            }
            return this.props.editable;
        },
        get displayData() {
            var filteredData = this.filterHandlers().reduce(function (f, g) { return function (args) { return g(f(args)); }; })(this._loadedData || this.props.data);
            return this.paginateFilter(filteredData);
        },
        get headers() {
            return this.props.headers && this.props.headers.map(function (header, index) { return new header_1.default(header, index); });
        },
        get fields() {
            return this.props.fields.map(function (field, index) { return new field_1.default(field, index); });
        },
        loadByUrl: function () {
            return tslib_1.__awaiter(this, void 0, void 0, function () {
                var res, e_1;
                return tslib_1.__generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (!this.props.url) {
                                return [2];
                            }
                            this.inProgress = true;
                            _a.label = 1;
                        case 1:
                            _a.trys.push([1, 3, , 4]);
                            return [4, http_1.load(this.props.url)];
                        case 2:
                            res = _a.sent();
                            this.inProgress = false;
                            this._loadedData = (this.props.fetchSuccess ? this.props.fetchSuccess(res) : res);
                            return [3, 4];
                        case 3:
                            e_1 = _a.sent();
                            this.error = e_1.message;
                            console.error(e_1);
                            return [3, 4];
                        case 4: return [2];
                    }
                });
            });
        },
    }); });
}
exports.useRootStore = useRootStore;
//# sourceMappingURL=index.js.map